const io = require('socket.io');
const AuthService = require('./AuthService');
const UpService = require('./UpService');
const adqGetDashboardFiliais = require('../model/adqGetDashboardFiliais');
// const sefaz = require('../model/sefaz');
const deliveryController = require('../controller/DeliveryController');
const saleController = require('../controller/SaleController');
const sefazController = require('../controller/SefazController');
const SystemMonitoramento = require('../controller/SystemMonitoramentoController');
const intervalMonitoring = require('../functions/IntervalMonitoring');
// const parametrizacoes = require('../model/parametrizacoes');

class SocketService {
  async start(http) {
    this.clientlist = {};
    this.browserlist = {};
    this.slavelist = await this.getDashboardFiliais();

    this.parametersError = [];
    this.vendasError = [];
    this.satError = [];
    this.io = io(http, { cors: '*' });

    this.io.on('connection', (socket) => {
      console.log('newconnection: ', socket.id);
      socket.emit('updateVersion', { serviceName: 'xxx' });

      socket.on('login', (data) => {
        this.login(socket, data);
      });

      socket.on('iniciaExportacao', (data) => {
        // let tabelas = data.payload.filter ( it => !!it.DATA && it.DATA.length > 2 )
        // console.log('')
        // console.log( ' ----------------------------------------------------------' )
        // console.log( '  FILIAL: ', data.CDFILIAL, '|', 'LOJA: ', data.CDLOJA, '|', 'CAIXA: ', data.CDCAIXA )
        // console.log( ' ',data.transactionId, '|', data.DTMOVTURCAIX )
        // console.log( ' ----------------------------------------------------------' )
        // tabelas.forEach( item => {
        //     if(!!item.DATA){
        //         console.log( ' ', item.TABLENAME+':', item.DATA ? JSON.parse(item.DATA).length : item )
        //
        //     }
        // })
        try {
          data.payload = JSON.parse(data.payload);
          UpService.setInsereServidor(data.CDFILIAL, data.CDCAIXA, data.CDLOJA, data.payload);
        } catch (e) {}

        //UpService.setInsereServidor(data.CDFILIAL, data.CDCAIXA, data.CDLOJA, data.payload)
      });

      socket.on('importar', (data) => {
        let target = this.slavelist[socket.email][data.CDFILIAL];
        this.io
          .to(target.id)
          .emit('command', { type: 'import', data: { dataUrl: '/API/getStructure' } });
      });

      socket.on('confirmation', (data) => {
        // console.log(data);
      });

      socket.on('disconnect', () => {
        const subsidiaryIndex = this.slavelist.findIndex((item) => {
          return item.socketId === socket.id;
        });

        if (this.slavelist[subsidiaryIndex]) {
          this.slavelist[subsidiaryIndex].situacao = 'Inativa';

          const portalList = Object.keys(this.clientlist).map((item) => {
            return this.clientlist[item].id;
          });
          if (portalList.length) {
            this.io.to(portalList).emit('login', {
              text: 'UNIDADE DESCONECTADA',
              error: false,
              message: {
                data: this.slavelist,
              },
            });
          }
        }
      });

      /**
       * Socket de Vendas
       */
      socket.on('sales', (data) => {
        saleController.salesSlave(this.vendasError).then((response) => {
          setInterval(() => {
            this.io.to(socket.id).emit('sales', {
              text: 'VENDAS',
              error: false,
              message: {
                data: response,
              },
            });
          }, intervalMonitoring(data));
        }).catch((error) => {
          console.log(error);
        });
      });

      /**
       * Socket de Delivery
       */
      socket.on('salesDelivery', (data) => {
        deliveryController.getAssistantDeliveryList().then((response) => {
          /** Lista de vendas Delivery */
          setInterval(() => {
            this.io.to(socket.id).emit('salesDelivery', {
              text: 'VENDAS',
              error: false,
              message: {
                data: response,
              },
            });
          }, intervalMonitoring(data));
        }).catch((error) => {
          console.log('Erro foi: ', error);
        });
      });

      /**
       * Socket de Sat
       */
      socket.on('sat', (data) => {
        setInterval(() => {
          this.io.to(socket.id).emit('sat', {
            text: 'SAT',
            error: false,
            message: {
              data: this.satError,
            },
          });
        }, intervalMonitoring(data));
      });

      /**
       * Socket de Parametros
       */
      socket.on('parameters', (data) => {
        // this.parameters(socket, data);
        setInterval(() => {
          this.io.to(socket.id).emit('parameters', {
            text: 'PARAMETERS',
            error: false,
            message: {
              data: this.parametersError,
            },
          });
        }, intervalMonitoring(data));
      });

      /**
       * Socket de Sefaz
       */
      socket.on('sefaz', (data) => {
        sefazController.getSefaz().then((response) => {
          setInterval(() => {
            this.io.to(socket.id).emit('sefaz', {
              text: 'SEFAZ',
              error: false,
              message: {
                data: response,
              },
            });
          }, intervalMonitoring(data));
        }).catch((error) => {
          console.log('Erro foi: ', error);
        });
      });

      /**
       * Socket de Parametrizações do sistema
       */
      socket.on('parametrizacoes', (data) => {
        this.parametrizacoes(socket, data);
      });

      /**
       * Socket de informações do sistema
       * @returns Erros do sistema (Falta de tabela de banco e dados)
      */
      socket.on('infoSystemMonitoramento', (data) => {
        SystemMonitoramento.info().then((response) => {
          this.io.to(socket.id).emit('infoSystemMonitoramento', {
            message: {
              data: response,
            },
          });
        }).catch((error) => {
          console.log(error);
        });
      });
    });
  }

  import(socket, data) {}

  login(socket, data) {
    // Não é uma filial que está conectando e sim um portal
    if (!data.CDFILIAL) {
      // TODO Autenticar esse cliente, senão desconectar ele

      let auth = AuthService.autenticar(data.email, data.password)
        .then((result) => {
          // console.log(result);
          // Se estiver autenticado coloca ele na lista simples de controle
          socket.email = data.email;

          this.clientlist[data.email] = socket;
          this.io
            .to(socket.id)
            .emit('confirmLogin', { error: false, message: 'Você está conectado', result: result });

          if (!data.slave) {
            this.io.to(socket.id).emit('login', {
              text: 'PORTAL CONECTADO',
              error: false,
              message: this.slavelist,
              result,
            });
          }
        })
        .catch((error) => {
          this.io.to(socket.id).emit('message', 'Você não é um usuário autorizado!');
          socket.disconnect();
          // console.log(error);
        });
    } else {
      // Uma filial se coneta ao master
      // TODO Autenticar esse cliente, senão desconectar ele
      let auth = AuthService.autenticar(data.email, '', data.CDFILIAL)
        .then((result) => {
          // Se estiver autenticado coloca ele na lista simples de controle
          socket.email = data.email;
          socket.CDFILIAL = data.CDFILIAL;

          const subsidiaryIndex = this.slavelist.findIndex((item) => {
            return item.codigo === data.CDFILIAL;
          });

          if (this.slavelist[subsidiaryIndex]) {
            this.slavelist[subsidiaryIndex].situacao = 'Ativo';
            this.slavelist[subsidiaryIndex].socketId = socket.id;

            const portalList = Object.keys(this.clientlist).map((item) => {
              return this.clientlist[item].id;
            });

            this.io.to(socket.id).emit('message', result);
            this.io.to(socket.id).emit('consultaSAT', {});

            if (portalList.length) {
              // console.log('servidor', portalList);
              this.io.to(portalList).emit('login', {
                text: 'UNIDADE CONECTADA',
                error: false,
                message: {
                  data: this.slavelist,
                },
                result,
              });
            }

            /**
             * Monitoramento das vendas
             * Faz a solicitação para o Slaver
            */
            this.io.to(portalList).emit('saleSlave', data);
            socket.on('saleSlaveError', (vendasError) => {
              this.vendasError = vendasError;
            });

            /**
             * Monitoramento dos Sats
             * Faz a solicitação para o Slaver
            */
            this.io.to(portalList).emit('satSlave', data);
            socket.on('satSlaveError', (satError) => {
              this.satError = satError;
            });

            /**
             * Monitoramento dos parametros
             * parameters
             */
            this.io.to(portalList).emit('parametersSlave', data);
            socket.on('parametersSlaveError', (parametersError) => {
              this.parametersError = parametersError;
            });
          }
        })
        .catch((error) => {
          this.io.to(socket.id).emit('message', 'Você não é um usuário autorizado!');
          socket.disconnect();
          // console.log(error);
        });
    }
  }

  async getDashboardFiliais() {
    const filiais = [];
    const { rows } = await adqGetDashboardFiliais.get({ MIN_ROW: 1, MAX_ROW: 10 });

    rows.forEach((element) => {
      filiais.push({
        id: element[0],
        codigo: element[0],
        name: element[1],
        situacao: 'Inativa',
        email: '',
        socketId: '',
        sat: false,
        vendas: false,
        websocket: '',
      });
    });

    return filiais;
  }
}

module.exports = new SocketService();
