import { Modal, ModalService } from '@zeedhi/common';

import { SocketEvent } from '@/socket/socket-event';
import {
  Config,
  IDictionary,
  IEventParam,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';

export class DeliveryController {
  public socket?: Socket;

  public title: any;

  private fullscreenModal!: Modal;

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson('delivery', true);
  }

  public destroyFullscreenModal() {
    this.fullscreenModal?.destroy();
  }

  public openFullscreenModal(row:any) {
    // console.warn(row);
    this.title = 'Monitoramento Delivery da ';
    this.title += row.row.name;
    this.title += ' - ';
    this.title += row.row.id;
    this.fullscreenModal.show();
  }

  public closeFullscreenModal() {
    this.fullscreenModal.hide();
  }

  /** Status do Vendas */
  public getConditionalStyleDelivery({ row }: IDictionary<any>) {
    if (row.delivery === 's') {
      return 'green';
    }
    return 'red';
  }

  /** Status do Vendas */
  public getConditionalMessageDelivery({ row }: IDictionary<any>) {
    return row.message;
  }

  /**
   * Método executado quando o grid é criado
   */
  public gridMounted({ component }: IEventParam<TekGrid>) {
    // inicializa o socket
    this.socket = io(Config.endPoint);

    this.socket.off(SocketEvent.CONNECTION)
      .emit('salesDelivery', {
        slave: false,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('salesDelivery', async (branc: IDictionary) => {
        await component.datasource.updateData(branc.message.vendas);
        await component.datasource.get();
      });
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {

  }
}
