import { IEventParam, Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Form, GridEditable, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { Highlight } from '@zeedhi/zd-highlight-common';

export class ParametrizacoesController {
  private paramEditModal!: Modal;

  public isNotInlineEditing = true;

  /**
   * Método executado quando o grid é criado
   */
  public async gridMounted() {
    this.reloadGrid();
		this.paramEditModal = await ModalService.createFromJson('parametrizacoes-edit', true);
  }

  private reloadGrid() {
    const grid = Metadata.getInstance<TekGrid>('gridParams');
    const rq = 'parametrizacoes';
    const conexao = process.env.VUE_APP_METADATA_END_POINT + rq;

    grid.datasource.loading = true;
    const req = Http.get(conexao);
    req.then((response) => {
      grid.datasource.updateData(response.data);
      grid.datasource.loading = false;
		}).catch(() => {
      grid.datasource.loading = false;
    });
  }

	public gridBeforeDestroy() {
		this.paramEditModal.destroy();
	}

	public async gridCellClick({ component, row } : any) {
		if (!component.editing) {
			await this.paramEditModal.show();
			const form = Metadata.getInstance<Form>('edit-modal-form');
      form.value = row;
      const codeComponent = Metadata.getInstance<Highlight>('component');
      codeComponent.code = row.component;
      const codeItem = Metadata.getInstance<Highlight>('item');
      codeItem.code = row.item;
		}
	}

	public closeParamEditModal() {
		this.paramEditModal.hide();
	}

  public onInlineEdit({ component }: IEventParam<GridEditable>) {
    this.isNotInlineEditing = !component.editing;
  }

  public onBeforeSave() {
    const grid = Metadata.getInstance<TekGrid>('gridParams');
    grid.getEditedRows().forEach((element) => {
      if (element) {
        const rq = `connect/portal/${element.id}`;
        const conexao = process.env.VUE_APP_METADATA_END_POINT + rq;
        Http.put(conexao, element);
      }
    });
  }

  public async updateInModal() {
    const form = Metadata.getInstance<Form>('edit-modal-form');
    const codeComponent = Metadata.getInstance<Highlight>('component');
    const codeItem = Metadata.getInstance<Highlight>('item');
    form.value.component = codeComponent.code;
    form.value.item = codeItem.code;

    const rq = `connect/portal/${form.value.id}`;
    const conexao = process.env.VUE_APP_METADATA_END_POINT + rq;

    LoadingService.show();
    Http.put(conexao, form.value).then(() => {
      LoadingService.hide();
      this.paramEditModal.hide();
      this.reloadGrid();
    }).catch(() => {
      LoadingService.hide();
    });
	}
}
